@@ -27,26 +27,19 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 
 #include "pixman-private.h"
 
 #define Alpha(x) ((x) >> 24)
 
-static void
-init_source_image (source_image_t *image)
-{
-    image->class = SOURCE_IMAGE_CLASS_UNKNOWN;
-}
-
-static pixman_bool_t
-init_gradient (gradient_t     *gradient,
-	       const pixman_gradient_stop_t *stops,
-	       int	       n_stops)
+pixman_bool_t
+_pixman_init_gradient (gradient_t     *gradient,
+		       const pixman_gradient_stop_t *stops,
+		       int	       n_stops)
 {
     return_val_if_fail (n_stops > 0, FALSE);
 
-    init_source_image (&gradient->common);
-
     gradient->stops = pixman_malloc_ab (n_stops, sizeof (pixman_gradient_stop_t));
     if (!gradient->stops)
 	return FALSE;
@@ -58,22 +51,45 @@ init_gradient (gradient_t     *gradient,
     gradient->stop_range = 0xffff;
     gradient->color_table = NULL;
     gradient->color_table_size = 0;
+    gradient->common.class = SOURCE_IMAGE_CLASS_UNKNOWN;
 
     return TRUE;
 }
 
-static uint32_t
-color_to_uint32 (const pixman_color_t *color)
+/*
+ * By default, just evaluate the image at 32bpp and expand.  Individual image
+ * types can plug in a better scanline getter if they want to. For example
+ * we  could produce smoother gradients by evaluating them at higher color depth, but
+ * that's a project for the future.
+ */
+void
+_pixman_image_get_scanline_64_generic (pixman_image_t * pict, int x, int y, int width,
+				       uint64_t *buffer, uint64_t *mask, uint32_t maskBits)
 {
-    return
-	(color->alpha >> 8 << 24) |
-	(color->red >> 8 << 16) |
-        (color->green & 0xff00) |
-	(color->blue >> 8);
+    uint32_t *mask8 = NULL;
+
+    // Contract the mask image, if one exists, so that the 32-bit fetch function
+    // can use it.
+    if (mask) {
+        mask8 = pixman_malloc_ab(width, sizeof(uint32_t));
+	if (!mask8)
+	    return;
+	
+        pixman_contract(mask8, mask, width);
+    }
+
+    // Fetch the source image into the first half of buffer.
+    _pixman_image_get_scanline_32 (pict, x, y, width, (uint32_t*)buffer, mask8,
+				   maskBits);
+
+    // Expand from 32bpp to 64bpp in place.
+    pixman_expand(buffer, (uint32_t*)buffer, PIXMAN_a8r8g8b8, width);
+
+    free(mask8);
 }
 
-static pixman_image_t *
-allocate_image (void)
+pixman_image_t *
+_pixman_image_allocate (void)
 {
     pixman_image_t *image = malloc (sizeof (pixman_image_t));
 
@@ -95,11 +111,114 @@ allocate_image (void)
 	common->ref_count = 1;
 	common->read_func = NULL;
 	common->write_func = NULL;
+	common->classify = NULL;
     }
 
     return image;
 }
 
+source_pict_class_t
+_pixman_image_classify (pixman_image_t *image,
+			int             x,
+			int             y,
+			int             width,
+			int             height)
+{
+    if (image->common.classify)
+	return image->common.classify (image, x, y, width, height);
+    else
+	return SOURCE_IMAGE_CLASS_UNKNOWN;
+}
+
+void
+_pixman_image_get_scanline_32 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *mask, uint32_t mask_bits)
+{
+    image->common.get_scanline_32 (image, x, y, width, buffer, mask, mask_bits);
+}
+
+void
+_pixman_image_get_scanline_64 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *unused, uint32_t unused2)
+{
+    image->common.get_scanline_64 (image, x, y, width, buffer, unused, unused2);
+}
+
+/* Even thought the type of buffer is uint32_t *, the function actually expects
+ * a uint64_t *buffer.
+ */
+
+scanFetchProc
+_pixman_image_get_fetcher (pixman_image_t *image,
+			   int             wide)
+{
+    assert (image->common.get_scanline_64);
+    assert (image->common.get_scanline_32);
+    
+    if (wide)
+	return image->common.get_scanline_64;
+    else
+	return image->common.get_scanline_32;
+}
+
+#define WRITE_ACCESS(f) ((image->common.write_func)? f##_accessors : f)
+
+static void
+fbStore(bits_image_t * image, int x, int y, int width, uint32_t *buffer)
+{
+    uint32_t *bits;
+    int32_t stride;
+    storeProc32 store = WRITE_ACCESS(pixman_storeProcForPicture32)(image);
+    const pixman_indexed_t * indexed = image->indexed;
+
+    bits = image->bits;
+    stride = image->rowstride;
+    bits += y*stride;
+    store((pixman_image_t *)image, bits, buffer, x, width, indexed);
+}
+
+static void
+fbStore64(bits_image_t * image, int x, int y, int width, uint64_t *buffer)
+{
+    uint32_t *bits;
+    int32_t stride;
+    storeProc64 store = WRITE_ACCESS(pixman_storeProcForPicture64)(image);
+    const pixman_indexed_t * indexed = image->indexed;
+
+    bits = image->bits;
+    stride = image->rowstride;
+    bits += y*stride;
+    store((pixman_image_t *)image, bits, buffer, x, width, indexed);
+}
+
+scanStoreProc
+_pixman_image_get_storer (pixman_image_t *image,
+			  int             wide)
+{
+    if (image->common.alpha_map)
+    {
+	if (wide)
+	    return (scanStoreProc)WRITE_ACCESS(fbStoreExternalAlpha64);
+	else
+	    return (scanStoreProc)WRITE_ACCESS(fbStoreExternalAlpha);
+    }
+    else
+    {
+	if (wide)
+	    return (scanStoreProc)fbStore64;
+	else
+	    return (scanStoreProc)fbStore;
+    }
+}
+
+static void
+image_property_changed (pixman_image_t *image)
+{
+    
+    
+    image->common.property_changed (image);
+}
+
 /* Ref Counting */
 PIXMAN_EXPORT pixman_image_t *
 pixman_image_ref (pixman_image_t *image)
@@ -158,170 +277,9 @@ pixman_image_unref (pixman_image_t *image)
 }
 
 /* Constructors */
-PIXMAN_EXPORT pixman_image_t *
-pixman_image_create_solid_fill (pixman_color_t *color)
-{
-    pixman_image_t *img = allocate_image();
-    if (!img)
-	return NULL;
 
-    init_source_image (&img->solid.common);
-
-    img->type = SOLID;
-    img->solid.color = color_to_uint32 (color);
-
-    return img;
-}
-
-PIXMAN_EXPORT pixman_image_t *
-pixman_image_create_linear_gradient (pixman_point_fixed_t         *p1,
-				     pixman_point_fixed_t         *p2,
-				     const pixman_gradient_stop_t *stops,
-				     int                           n_stops)
-{
-    pixman_image_t *image;
-    linear_gradient_t *linear;
-
-    return_val_if_fail (n_stops >= 2, NULL);
-
-    image = allocate_image();
-
-    if (!image)
-	return NULL;
-
-    linear = &image->linear;
-
-    if (!init_gradient (&linear->common, stops, n_stops))
-    {
-	free (image);
-	return NULL;
-    }
-
-    linear->p1 = *p1;
-    linear->p2 = *p2;
-
-    image->type = LINEAR;
-
-    return image;
-}
-
-
-PIXMAN_EXPORT pixman_image_t *
-pixman_image_create_radial_gradient (pixman_point_fixed_t         *inner,
-				     pixman_point_fixed_t         *outer,
-				     pixman_fixed_t                inner_radius,
-				     pixman_fixed_t                outer_radius,
-				     const pixman_gradient_stop_t *stops,
-				     int                           n_stops)
-{
-    pixman_image_t *image;
-    radial_gradient_t *radial;
-
-    return_val_if_fail (n_stops >= 2, NULL);
-
-    image = allocate_image();
-
-    if (!image)
-	return NULL;
-
-    radial = &image->radial;
-
-    if (!init_gradient (&radial->common, stops, n_stops))
-    {
-	free (image);
-	return NULL;
-    }
-
-    image->type = RADIAL;
-
-    radial->c1.x = inner->x;
-    radial->c1.y = inner->y;
-    radial->c1.radius = inner_radius;
-    radial->c2.x = outer->x;
-    radial->c2.y = outer->y;
-    radial->c2.radius = outer_radius;
-    radial->cdx = pixman_fixed_to_double (radial->c2.x - radial->c1.x);
-    radial->cdy = pixman_fixed_to_double (radial->c2.y - radial->c1.y);
-    radial->dr = pixman_fixed_to_double (radial->c2.radius - radial->c1.radius);
-    radial->A = (radial->cdx * radial->cdx
-		 + radial->cdy * radial->cdy
-		 - radial->dr  * radial->dr);
-
-    return image;
-}
-
-PIXMAN_EXPORT pixman_image_t *
-pixman_image_create_conical_gradient (pixman_point_fixed_t *center,
-				      pixman_fixed_t angle,
-				      const pixman_gradient_stop_t *stops,
-				      int n_stops)
-{
-    pixman_image_t *image = allocate_image();
-    conical_gradient_t *conical;
-
-    if (!image)
-	return NULL;
-
-    conical = &image->conical;
-
-    if (!init_gradient (&conical->common, stops, n_stops))
-    {
-	free (image);
-	return NULL;
-    }
-
-    image->type = CONICAL;
-    conical->center = *center;
-    conical->angle = angle;
-
-    return image;
-}
-
-static uint32_t *
-create_bits (pixman_format_code_t format,
-	     int		  width,
-	     int		  height,
-	     int		 *rowstride_bytes)
-{
-    int stride;
-    int buf_size;
-    int bpp;
-
-    /* what follows is a long-winded way, avoiding any possibility of integer
-     * overflows, of saying:
-     * stride = ((width * bpp + FB_MASK) >> FB_SHIFT) * sizeof (uint32_t);
-     */
-
-    bpp = PIXMAN_FORMAT_BPP (format);
-    if (pixman_multiply_overflows_int (width, bpp))
-	return NULL;
-
-    stride = width * bpp;
-    if (pixman_addition_overflows_int (stride, FB_MASK))
-	return NULL;
-
-    stride += FB_MASK;
-    stride >>= FB_SHIFT;
-
-#if FB_SHIFT < 2
-    if (pixman_multiply_overflows_int (stride, sizeof (uint32_t)))
-	return NULL;
-#endif
-    stride *= sizeof (uint32_t);
-
-    if (pixman_multiply_overflows_int (height, stride))
-	return NULL;
-
-    buf_size = height * stride;
-
-    if (rowstride_bytes)
-	*rowstride_bytes = stride;
-
-    return calloc (buf_size, 1);
-}
-
-static void
-reset_clip_region (pixman_image_t *image)
+void
+_pixman_image_reset_clip_region (pixman_image_t *image)
 {
     pixman_region32_fini (&image->common.clip_region);
 
@@ -336,72 +294,27 @@ reset_clip_region (pixman_image_t *image)
     }
 }
 
-PIXMAN_EXPORT pixman_image_t *
-pixman_image_create_bits (pixman_format_code_t  format,
-			  int                   width,
-			  int                   height,
-			  uint32_t	       *bits,
-			  int			rowstride_bytes)
-{
-    pixman_image_t *image;
-    uint32_t *free_me = NULL;
-
-    /* must be a whole number of uint32_t's
-     */
-    return_val_if_fail (bits == NULL ||
-			(rowstride_bytes % sizeof (uint32_t)) == 0, NULL);
-
-    if (!bits && width && height)
-    {
-	free_me = bits = create_bits (format, width, height, &rowstride_bytes);
-	if (!bits)
-	    return NULL;
-    }
-
-    image = allocate_image();
-
-    if (!image) {
-	if (free_me)
-	    free (free_me);
-	return NULL;
-    }
-    
-    image->type = BITS;
-    image->bits.format = format;
-    image->bits.width = width;
-    image->bits.height = height;
-    image->bits.bits = bits;
-    image->bits.free_me = free_me;
-
-    image->bits.rowstride = rowstride_bytes / (int) sizeof (uint32_t); /* we store it in number
-								  * of uint32_t's
-								  */
-    image->bits.indexed = NULL;
-
-    pixman_region32_fini (&image->common.full_region);
-    pixman_region32_init_rect (&image->common.full_region, 0, 0,
-			       image->bits.width, image->bits.height);
-
-    reset_clip_region (image);
-    return image;
-}
-
 PIXMAN_EXPORT pixman_bool_t
 pixman_image_set_clip_region32 (pixman_image_t *image,
 				pixman_region32_t *region)
 {
     image_common_t *common = (image_common_t *)image;
+    pixman_bool_t result;
 
     if (region)
     {
-	return pixman_region32_copy (&common->clip_region, region);
+	result = pixman_region32_copy (&common->clip_region, region);
     }
     else
     {
-	reset_clip_region (image);
+	_pixman_image_reset_clip_region (image);
 
-	return TRUE;
+	result = TRUE;
     }
+
+    image_property_changed (image);
+
+    return result;
 }
 
 
@@ -410,17 +323,22 @@ pixman_image_set_clip_region (pixman_image_t    *image,
 			      pixman_region16_t *region)
 {
     image_common_t *common = (image_common_t *)image;
+    pixman_bool_t result;
 
     if (region)
     {
-	return pixman_region32_copy_from_region16 (&common->clip_region, region);
+	result = pixman_region32_copy_from_region16 (&common->clip_region, region);
     }
     else
     {
-	reset_clip_region (image);
+	_pixman_image_reset_clip_region (image);
 
-	return TRUE;
+	result = TRUE;
     }
+
+    image_property_changed (image);
+
+    return result;
 }
 
 /* Sets whether the clip region includes a clip region set by the client
@@ -430,6 +348,8 @@ pixman_image_set_has_client_clip (pixman_image_t *image,
 				  pixman_bool_t	  client_clip)
 {
     image->common.has_client_clip = client_clip;
+
+    image_property_changed (image);
 }
 
 PIXMAN_EXPORT pixman_bool_t
@@ -445,6 +365,7 @@ pixman_image_set_transform (pixman_image_t           *image,
     };
 
     image_common_t *common = (image_common_t *)image;
+    pixman_bool_t result;
 
     if (common->transform == transform)
 	return TRUE;
@@ -453,16 +374,24 @@ pixman_image_set_transform (pixman_image_t           *image,
     {
 	free(common->transform);
 	common->transform = NULL;
-	return TRUE;
+	result = TRUE;
+	goto out;
     }
 
     if (common->transform == NULL)
 	common->transform = malloc (sizeof (pixman_transform_t));
+
     if (common->transform == NULL)
-	return FALSE;
+    {
+	result = FALSE;
+	goto out;
+    }
 
     memcpy(common->transform, transform, sizeof(pixman_transform_t));
 
+out:
+    image_property_changed (image);
+    
     return TRUE;
 }
 
@@ -471,6 +400,8 @@ pixman_image_set_repeat (pixman_image_t  *image,
 			 pixman_repeat_t  repeat)
 {
     image->common.repeat = repeat;
+
+    image_property_changed (image);
 }
 
 PIXMAN_EXPORT pixman_bool_t
@@ -503,6 +434,8 @@ pixman_image_set_filter (pixman_image_t       *image,
 
     common->filter_params = new_params;
     common->n_filter_params = n_params;
+
+    image_property_changed (image);
     return TRUE;
 }
 
@@ -516,6 +449,8 @@ pixman_image_set_source_clipping (pixman_image_t  *image,
 	common->src_clip = &common->clip_region;
     else
 	common->src_clip = &common->full_region;
+
+    image_property_changed (image);
 }
 
 /* Unlike all the other property setters, this function does not
@@ -529,6 +464,8 @@ pixman_image_set_indexed (pixman_image_t	 *image,
     bits_image_t *bits = (bits_image_t *)image;
 
     bits->indexed = indexed;
+
+    image_property_changed (image);
 }
 
 PIXMAN_EXPORT void
@@ -554,6 +491,8 @@ pixman_image_set_alpha_map (pixman_image_t *image,
 
     common->alpha_origin.x = x;
     common->alpha_origin.y = y;
+
+    image_property_changed (image);
 }
 
 PIXMAN_EXPORT void
@@ -561,6 +500,8 @@ pixman_image_set_component_alpha   (pixman_image_t       *image,
 				    pixman_bool_t         component_alpha)
 {
     image->common.component_alpha = component_alpha;
+
+    image_property_changed (image);
 }
 
 
@@ -573,6 +514,8 @@ pixman_image_set_accessors (pixman_image_t             *image,
 
     image->common.read_func = read_func;
     image->common.write_func = write_func;
+
+    image_property_changed (image);
 }
 
 PIXMAN_EXPORT uint32_t *
@@ -620,6 +563,16 @@ pixman_image_get_depth (pixman_image_t *image)
     return 0;
 }
 
+static uint32_t
+color_to_uint32 (const pixman_color_t *color)
+{
+    return
+	(color->alpha >> 8 << 24) |
+	(color->red >> 8 << 16) |
+        (color->green & 0xff00) |
+	(color->blue >> 8);
+}
+
 static pixman_bool_t
 color_to_pixel (pixman_color_t *color,
 		uint32_t       *pixel,
